// package ir.idehpayam.api.sample;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.hc.client5.http.classic.methods.HttpGet;
import org.apache.hc.client5.http.classic.methods.HttpPost;
import org.apache.hc.client5.http.entity.UrlEncodedFormEntity;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.CloseableHttpResponse;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.HttpEntity;
import org.apache.hc.core5.http.NameValuePair;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.apache.hc.core5.http.message.BasicNameValuePair;
import org.apache.hc.core5.http.io.entity.StringEntity;
import org.json.simple.JSONObject;
import org.json.simple.JSONArray;
import org.json.simple.parser.ParseException;
import org.json.simple.parser.JSONParser;

public class IdehPayamApiClient
{
    private class IdehPayamApiAdapter
    {
        public static final int TYPE_SOAP = 1;
        public static final int TYPE_REST = 2;

        private static final String restUrl = "http://localhost:8800/idehpayam/new-api/rest.php";
        private static final String soapUrl = "http://localhost:8800/idehpayam/new-api/soap.php?wsdl";

        private int _type = 1;
        private String _username = "";
        private String _password = "";
        // private Object _client = null;
        private String _lastError = "";
        private String _lastResponse = "";

        public IdehPayamApiAdapter(String username, String password, int type) throws Exception
        {
            if (type != TYPE_SOAP && type != TYPE_REST) {
                throw new Exception("type parameter out of bound");
            }

            _username = username;
            _password = password;
            _type = type;

            switch (type) {
                case TYPE_SOAP:
                break;
                case TYPE_REST:
                break;
            }
        }

        public String GetLastError()
        {
            return _lastError;
        }

        public String GetLastResponse()
        {
            return _lastResponse;
        }

        public long GetUserCredit()
        {
            long credit = 0;

            switch (_type) {
                case TYPE_SOAP:
                    try {
                        /*$response = $this->_client->__call('GetUserCredit', [
                            "username" => $this->_username,
                            "password" => $this->_password
                        ]);
            
                        if (isset($response->status)) {
                            switch ($response->status) {
                                case 0:
                                    return $response->value;
                                break;
                            }
                        } else {
                            $this->_lastError = 'Server error: status does not set.';
                        }*/
                    } catch (Exception exp) {
                        _lastError = "Server error: " + exp.toString();
                    }
                break;
                case TYPE_REST:
                    try (final CloseableHttpClient httpclient = HttpClients.createDefault()) {
                        final HttpPost httpPost = new HttpPost(restUrl);
                        StringEntity params = new StringEntity("{\"operation\":\"GetUserCredit\",\"username\":\"" + _username + "\", \"password\":\"" + _password + "\"}");
                        httpPost.setEntity(params);

                        try (final CloseableHttpResponse response = httpclient.execute(httpPost)) {
                            // System.out.println(response.getCode() + " " + response.getReasonPhrase());
                            final HttpEntity entity = response.getEntity();
                            String responseString = EntityUtils.toString(entity);
                            _lastResponse = responseString;
                            JSONParser parser = new JSONParser();
                            JSONObject jsonObject = (JSONObject)parser.parse(responseString);
                            Long status = (Long)jsonObject.get("status");
                            if (status == null) {
                                _lastError = "Server error: status does not set.";
                            } else {
                                credit = (long)((JSONObject)jsonObject.get("result")).get("value");
                            }
                            EntityUtils.consume(entity);
                        } catch (Exception exp) {
                            System.out.println(exp);
                            _lastError = "Server error: " + exp.toString();
                        }
                    } catch (Exception httpExp) {
                        System.out.println(httpExp);
                        _lastError = "Server error: " + httpExp.toString();
                    }
                break;
            }

            return credit;
        }
    }

    public static void main(String[] args) throws IOException, InterruptedException
    {
        try {
            IdehPayamApiAdapter adapter = new IdehPayamApiClient().new IdehPayamApiAdapter("prodo", "123456", IdehPayamApiAdapter.TYPE_SOAP);
            long credit = adapter.GetUserCredit();
            System.out.println(credit);
        } catch (Exception exp) {
            System.out.println(exp.toString());
        }
    }
}
